#! /usr/bin/env python
# encoding: utf-8

from __future__ import absolute_import

import os
import io
import json
import requests

from .client import CoinsBillClient

try:
    basestring
except NameError:
    basestring = str

class ListResource(CoinsBillClient):

    def get(self, id=None):
        path = self.ENDPOINT if id is None else '%s/%s' % (self.ENDPOINT, id)

        return self._request('GET', path)

class CreateResource(CoinsBillClient):

    def create(self, **params):
        uri = self.BASE_URL + self.ENDPOINT
        data = dict(**(params or {}))
        r = self._get_response(uri, data)

        return r if r.status_code == 201 else self._get_response(uri, data)

    def _get_response(self, uri, data):
        return self.client.post(uri, data=json.dumps(data))

class UpdateResource(CoinsBillClient):

    def update(self, id=None, **params):
        path = self.ENDPOINT if id is None else '%s/%s' % (self.ENDPOINT, id)
        uri = self.BASE_URL + path
        data = dict(**(params or {}))
        
        r  = self.client.put(uri, data=json.dumps(data))
        return r

    def _get_response(self, uri, data):
        return self.client.post(uri, data=json.dumps(data))

class Invoice(ListResource, CreateResource, UpdateResource):
    """Handle Invoice to the CoinsBill API."""
    ENDPOINT = 'invoice'

class Button(ListResource, CreateResource, UpdateResource):
    """Handle Payment Buttons to the CoinsBill API."""
    ENDPOINT = 'buttons'

class Customer(ListResource):
    """Handle Customer to the CoinsBill API."""
    ENDPOINT = 'customer'

class Settlement(ListResource):
    """Handle Payment Settlement to the CoinsBill API."""
    ENDPOINT = 'settlement'

class Rates(ListResource):
    """Handle Exchange Rates to the CoinsBill API."""
    ENDPOINT = 'rates'

class ResourceMixin(Invoice):
    """Combined Mixin API."""
    pass


class CoinsBill:

    def __init__(self, api_key):
        self.invoice = Invoice(api_key)
        self.button = Button(api_key)
        self.customer = Customer(api_key)
        self.settlement = Settlement(api_key)
        self.rates = Rates(api_key)

    def invoice(self):
        return self.invoice

    def button(self):
        return self.button

    def customer(self):
        return self.customer

    def settlement(self):
        return self.settlement

    def rates(self):
        return self.rates


import calendar
from datetime import datetime


def timestamp_to_datetime(ts):
    return datetime.utcfromtimestamp(float(ts))


def datetime_to_timestamp(dt):
    return calendar.timegm(dt.timetuple())



#! /usr/bin/env python
# encoding: utf-8

from __future__ import absolute_import

__version__ = "1.0.1"

api_key = None
api_base = 'https://www.coinsbill.com/api/'

from .client import CoinsBillClient 

from .resource import ( 
    Invoice,
    CoinsBill
    )


import json

from base import APIClient


class CoinsBillError(Exception):
    STATUS_MAP = {
        200: "OK: Success",
        202: "Accepted: Your request was accepted and the user was queued for processing.",
        401: "Not Authorized: either you need to provide authentication credentials, or the credentials provided aren't valid.",
        403: "Bad Request: your request is invalid, and we'll return an error message that tells you why. This is the status code returned if you've exceeded the rate limit (see below).",
        404: "Not Found: either you're requesting an invalid URI or the resource in question doesn't exist (ex: no such user in our system).",
        500: "Internal Server Error: we did something wrong.",
        501: "Not implemented.",
        502: "Bad Gateway: returned if CoinsBill is down or being upgraded.",
        503: "Service Unavailable: the CoinsBill servers are up, but are overloaded with requests. Try again later.",
    }

    def __init__(self, status, response=None):
        self.status = status
        self.response = response

    def __str__(self):
        return "%s (%s)" % (self.status, self.STATUS_MAP.get(self.status, 'Unknown error.'))

    def __repr__(self):
        return "%s(status=%s)" % (self.__class__.__name__, self.status)


class CoinsBillClient(APIClient):
    """
    """
    
    def _handle_response(self, response):
        if response.status_code > 299:
            raise CoinsBillError(response.status_code, response=response)

        return super(CoinsBillClient, self)._handle_response(response)

    def call(self, path, **params):
        return self._request('GET', path, params=params)

    






import json
import requests

import coinsbill

class APIClient(object):
    BASE_URL = 'https://www.coinsbill.com/api/'
    
    def __init__(self, api_key=None, rate_limit_lock=None):
        key = api_key if api_key is not None else coinsbill.api_key
        self.client = requests.session()
        self.client.headers = {
            "Authorization": "bearer %s" % key,
            "Content-Type": "application/json",
        } 
        
        self.rate_limit_lock = rate_limit_lock

    def _compose_url(self, path, params=None):
        
        return self.BASE_URL + path 

    def _handle_response(self, response):
        
        return response


    def status(self, response):
        return response.status_code

    def _request(self, method, path, **params ):
        uri = self._compose_url(path, params)
        
        self.rate_limit_lock and self.rate_limit_lock.acquire()

        data = dict(**(params or {}))

        if params:
            data.update(params)    
        
        if method == "POST":
            response = self.client.post(uri, data=json.dumps(data))
            
        elif method == "PUT":    
            response = self.client.put(uri, data=json.dumps(data))
            
        elif method == "GET": 
            response = self.client.get(uri, data=json.dumps(data))
            
        else:
            pass

        return self._handle_response(response)

    def _get(self, path, **params):
        return self._request('GET', path, **params)

    def _create(self, path, **params):
        return self._request('POST', path, **params)

    def _update(self, path, **params):
        return self._request('PUT', path, **params)

    def _call(self, path, **params):
        return self._request('GET', path, params=params)

    





#! /usr/bin/env python
# encoding: utf-8

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))



setup(name='coinsbill',
    version='1.0.8',
    description='Python interaction with the CoinsBill API.',
    url='https://www.coinsbill.com/developers',
    author='CoinsBill',
    author_email='support@coinsbill.com',
    license='Apache License, Version 2.0, January 2004',
    packages=['coinsbill', ],
    install_requires=['requests>=2.4.0'],
    keywords = ['coinsbill', 'bitcoin', 'payment'], 
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)

